/*
 * File:  ntag.c
 * Use:   ntag driver routines
 * Autjor:  Jeff Seaman
 * Project: ntage & ble demo
 * Processor: nRF52840
 * SDK:       nRF5_SDK_16.0.0
 */
#include "nrf_drv_twi.h"

#include "nrf_delay.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
/* TWI instance ID. */
#if TWI0_ENABLED
#define TWI_INSTANCE_ID 0
#elif TWI1_ENABLED
#define TWI_INSTANCE_ID 1
#endif

/* Number of possible TWI addresses. */
#define TWI_ADDRESSES 127
#define NTAG_SLAVE_ADDR 0x55
/* TWI instance. */
static const nrf_drv_twi_t m_twi = NRF_DRV_TWI_INSTANCE(TWI_INSTANCE_ID);

//
// read a block of 16 values from the ntag eeprom
// @param block - the memory block to read
// @param buffer - the buffer for the values read

void read_ntag(int block, uint8_t *buffer) {

  ret_code_t err_code;
  uint8_t ntag_reg = block;
  err_code = nrf_drv_twi_tx(&m_twi, NTAG_SLAVE_ADDR, &ntag_reg, 1, false);

  err_code = nrf_drv_twi_rx(&m_twi, NTAG_SLAVE_ADDR, buffer, 16);
}

//
// write a block of 16 values to the ntag eeprom
// @param block - the memory block to read
// @param buffer - the buffer for the values to write

void write_ntag(uint8_t block, uint8_t *buffer) {
  uint8_t write_buffer[24];
  ret_code_t err_code;
  write_buffer[0] = block;
  for (int k = 0; k < 16; ++k)
    write_buffer[k + 1] = buffer[k];
  err_code = nrf_drv_twi_tx(&m_twi, NTAG_SLAVE_ADDR, write_buffer, 17, true);
}

/**
 * @brief TWI initialization.
 */
void twi_init(void) {
  ret_code_t err_code;

  const nrf_drv_twi_config_t twi_config = {
      .scl = 27, // 27
      .sda = 26, // 26
      .frequency = NRF_DRV_TWI_FREQ_100K,
      .interrupt_priority = APP_IRQ_PRIORITY_HIGH,
      .clear_bus_init = false};

  err_code = nrf_drv_twi_init(&m_twi, &twi_config, NULL, NULL);
  APP_ERROR_CHECK(err_code);

  nrf_drv_twi_enable(&m_twi);
}

/*
 * locate the ntag chip by checking all TWI addresses
 */
uint8_t l_ntag(void) {
  ret_code_t err_code;
  uint8_t cnt = 1;
  uint8_t buffy[16];
  uint8_t ntag_reg;
  for (cnt = 1; cnt <= 127; ++cnt) {
    // write the address to read
    ntag_reg = 0;
    err_code = nrf_drv_twi_tx(&m_twi, cnt, &ntag_reg, 1, false);
    // read the data
    buffy[0] = 0;
    err_code = nrf_drv_twi_rx(&m_twi, cnt, buffy, 16);
    if(!err_code)  { 
  //      printf("mytest");
  //  printf("Checking ADDRESS 0x%02X, status: %02X, buff[0]: %02X\r\n", cnt, err_code, buffy[0]);
    delay_20ms();
    }
  }
  return 0;
}

////////////////////////////////
// this is a 20 ms delay at 2 MHz for the ntag wirtes
void delay_20ms(void) {
  nrf_delay_ms(20);
}

// read a session register
// @param rega - the status register to be read
// @param regdata - variable for register value
void read_register(uint8_t rega, uint8_t *regdata) {
ret_code_t err_code;
  uint8_t txBuff[8];
  txBuff[0] = 0xfe; // set the status MEMA register address
  txBuff[1] = rega; // select the register to read
                    // write the register info

  err_code = nrf_drv_twi_tx(&m_twi, NTAG_SLAVE_ADDR, txBuff, 2, false);
  // read back the value in the register
  err_code = nrf_drv_twi_rx(&m_twi, NTAG_SLAVE_ADDR, regdata, 1);
}

// write a value to a session register
// @param rega - the status register to be written
// @param mask - the bit mask to be written
// @param reg-data - the value to be written

void write_register(uint8_t rega, uint8_t mask, uint8_t reg_data) {
  ret_code_t err_code;
  uint8_t txBuff[8];

  txBuff[0] = 0xfe;     // set NEMA  0xfe  = status register
  txBuff[1] = rega;     // set REGA
  txBuff[2] = mask;     // set the bit mask
  txBuff[3] = reg_data; // the data value

  // write the register info
  err_code = nrf_drv_twi_tx(&m_twi, NTAG_SLAVE_ADDR, txBuff, 4, true);
}



// read a bit from a session register
// @param status_rega - the status register to be read
// @param bit - the bit in the register
// @return the value of the bit 0 or 1

uint8_t read_session_bit(uint8_t status_reg, uint8_t bit) {
  uint8_t mask; // value for the bit mask in the register shifted 1
  uint8_t val;  // value for the bit in the register shifted 1 or 0
  uint8_t ret_val;
  mask = 1 << bit;
  read_register(status_reg, &val);

  ret_val = (val && mask) >> bit;
  return ret_val;
}

// write a bit from a session register
// @param status_rega - the status register to be read
// @param bit - the bit in the register
// @@param set -  the value of the bit 0 or 1
void write_session_bit(uint8_t status_reg, uint8_t bit, uint8_t set) {
  uint8_t mask; // value for the bit mask in the register shifted 1
  uint8_t val;  // value for the bit in the register shifted 1 or 0
  mask = 1 << bit;
  if (set)
    val = 1 << bit;
  else
    val = 0;
  write_register(status_reg, mask, val);
}